/*
 *      X Window System Interface  griface.c                
 *      (Xlib functions are called from this module only)
 *
 *      Copyright (c) 1994,1995  T.Hruz, R.Gosiorovsky, I.Povazan
 *
 *   from Lab-3D 0.8
 *
 *   Authors: T.Hruz, I. Povazan - algorithms specification,
 *                                 hidden line, hidden surface algorithms
 *            R. Gosiorovsky     - Lab-3D interactive environment,
 *                                 implementation
 *
 * Copyright (C) 1994,1995 by authors
 *
 * This software may be freely copied, modified and redistributed
 * without fee provided that this copyright notice is preserved
 * intact on all copies and modified copies.
 *
 * There is no warranty or other guarantee of fitness of this software.
 * It is provided solely "as is". The authors disclaim all
 * responsibility and liability with respect to this software's usage
 * or its effect upon hardware or computer systems.
 *
 */

#include "typy.h"
#include <X11/Xlib.h>
#include <X11/cursorfont.h>

VOID X_FillPoly(INT N,POINT2 point[],INT CO,INT CV);


/*
 *  X-related variables
 */

Display             *disp;
Window               root,win;
GC                   gc;
INT                  scr;
XEvent               event;
KeySym               keysym;
XWindowAttributes    watt;
XSetWindowAttributes satt;
Drawable             DRAW;
Pixmap               menumap,map;
Cursor               cursor1,cursor2;
Colormap             cmap,dmap;
XImage              *image;
INT                  depth;

CHAR                *Idata;         /*  image data       */
INT                 Image_F;        /*  color for image  */
INT                 Cur_Out;        /*  current output   */
FILE                *output_file; 


/*
 *     Set Palette 
 */

VOID Palette(int ncols)
  {
     INT i;  
     static XColor ccc;
     static COLOR color;
     f= &color;

     dmap = XDefaultColormap(disp,scr);

     for(i=0;i<ncols;i++)
       {
          ccc.red   = 65536*i/ncols;
          ccc.green = 65536*i/ncols;
          ccc.blue  = 65536*i/ncols;   
          ccc.flags = DoRed|DoGreen|DoBlue;

          if (!XAllocColor(disp,dmap,&ccc))
            { 
               fprintf(stderr,"Can not alloc color %d:  colormap full\n",i);
               ccc.pixel=i;
            }   

          f->my_colors[i] = ccc.pixel;

/*          printf("%d %d \n",i,ccc.pixel);  */
      }

    f->my_colors[ncols] = 0x01;   /* used for XOR line mode */

    f->frame  = 0;
    f->shadow = 0;
    f->text   = 0;
    f->work   = 0;
    f->window = ncols-1;
    f->header_text= ncols-1;
    f->line   = ncols-2;
    f->bottom = ncols-2;
    f->header = ncols-2;
    f->border = ncols-2;

    f->xor    = ncols;

 }

/*
 *      Graphics startup Procedure
 */

VOID init_graph(int WX,int WY,int ncols)
  {

       if ((disp=XOpenDisplay(NULL))==NULL)
          { printf(" Can not open XDisplay \n Bye. \n"); exit(0); }

       if (WX<200) WX=600;
       if (WY<200) WY=400;
/*       printf("window size: %dx%d\n",WX,WY); */

       root=XDefaultRootWindow(disp);
       scr=XDefaultScreen(disp);
       gc=XDefaultGC(disp,scr);
 
       depth = XDefaultDepth(disp,scr);
/*       printf("depth of screen: %d\n",depth);  */
    
       cursor1 = XCreateFontCursor(disp,XC_arrow);
       cursor2 = XCreateFontCursor(disp,XC_tcross);

       satt.background_pixmap     = ParentRelative;
       satt.background_pixel      = 0x0L;
       satt.border_pixmap         = CopyFromParent;
       satt.border_pixel          = 0x0;
       satt.bit_gravity           = StaticGravity;;
       satt.win_gravity           = NorthWestGravity;   
       satt.backing_store         = Always;
       satt.backing_planes        = 0x1L;
       satt.backing_pixel         = 0x1L; 
       satt.save_under            = True;
       satt.event_mask            = KeyPressMask | ButtonPressMask |
                                    StructureNotifyMask;
       satt.do_not_propagate_mask = 0;
       satt.override_redirect     = False;
       satt.colormap              = CopyFromParent;
       satt.cursor                = cursor1;
 
       win=XCreateWindow(disp,root,0,0,WX,WY,8,depth,CopyFromParent,
                         XDefaultVisual(disp,scr),0xffffffffL,&satt);

         XStoreName(disp,win,"Xgrid"); 
       XSetIconName(disp,win,"Xgrid 0.01");

       XAutoRepeatOn(disp);
/*       XBell(disp,20);       */

       XMapWindow(disp,win);
       XSetLineAttributes(disp,gc,0,LineSolid,CapNotLast,JoinMiter);  
       
       Palette(ncols);    

       GColor(f->line);
       XSync(disp,1);
       DRAW = win;       /* could be 'map' also */
  }

VOID Gflush()
{
       XFlush(disp);
}

VOID close_graph()
  {
     XFlush(disp);
     XCloseDisplay(disp);
     printf("\n  Thanks for using Xgrid.\n\n");
  }

/*
 *     get window height and width
 */

INT GetMaxX()
  {
     XGetWindowAttributes(disp,win,&watt);
     return(watt.width);
  }
INT GetMaxY()
  {
     XGetWindowAttributes(disp,win,&watt); 
     return(watt.height);
  }

/*
 *    output device interface
 */

VOID GColor(INT Color) 
  {
     XSetForeground(disp,gc,f->my_colors[Color]);  
     if (Cur_Out==IMAGE) Image_F = f->my_colors[Color];
  }

/*
 *    point interface
 */

VOID GPoint(INT x,INT y)   /* image point */
  {
    XPutPixel(image,x,y,Image_F);  
  }

VOID FPoint(INT x,INT y)   /* file point */
  {
    fprintf(output_file,"%d %d\n",x,y);   
  }

VOID X_Point(INT x,INT y)   /* drawable point */
  {
     XDrawPoint(disp,DRAW,gc,x,y); 
  }

VOID X_Line(INT x1,INT y1,INT x2,INT y2,INT Color)
  {
     GColor(Color);
     XDrawLine(disp,win,gc,x1,y1,x2,y2);
  } 

VOID X_Poly(INT N,POINT2 point[])
  {
     XDrawLines(disp,DRAW,gc,(XPoint *)point,N,CoordModeOrigin);  
  } 

VOID X_FillPoly(INT N,POINT2 point[],INT CO,INT CV)
  {
    GColor(CV); 
    XFillPolygon(disp,DRAW,gc,(XPoint *)point,N,Convex,CoordModeOrigin);  

    if (CO) 
       { GColor(CO);
         point[N]=point[0];
         XDrawLines(disp,DRAW,gc,(XPoint *)point,N+1,CoordModeOrigin); }
  }  

VOID Line_mode(INT Mod)
  {
     if (Mod==0) XSetFunction(disp,gc,GXcopy);
            else XSetFunction(disp,gc,GXxor);   
  }

VOID FillRectangle(INT x1,INT y1,INT x2,INT y2,INT Color)
  {
      GColor(Color);
      XFillRectangle(disp,win,gc,x1,y1,x2-x1+1,y2-y1+1);
  }

VOID Rectangle(INT x1,INT y1,INT x2,INT y2,INT Color)
  {
      GColor(Color);
      XDrawRectangle(disp,win,gc,x1,y1,x2-x1,y2-y1);
  } 

/*
 *    Output Selection
 */


INT TextHeight(CHAR *Text)
  {
    return(12);
  }

INT TextWidth(CHAR *Text)
  {
    return(6*strlen(Text));
  } 

VOID TextXY(INT x,INT y,CHAR *Text)
  {
    INT Hi;
    Hi=TextHeight("I")-3;
    XDrawString(disp,win,gc,x,y+Hi,Text,strlen(Text));  
  } 

/*
 *      save under 
 */

VOID GetArea(INT x1,INT y1,INT x2,INT y2)
  { 
    XCopyArea(disp,win,menumap,gc,x1,y1,x2-x1+1,y2-y1+1,0,0);   
  }

VOID PutArea(INT x1,INT y1,INT x2,INT y2)
  {
    XCopyArea(disp,menumap,win,gc,0,0,x2-x1+1,y2-y1+1,x1,y1);  
  } 

/*
 *     Clear & Put
 */

VOID Clear()
  {
    INT i;
    INT x1,y1,x2,y2;
    
    x1=MinX;
    y1=MinY;
    x2=MaxX;
    y2=MaxY;

    GColor(f->work);
    if (Cur_Out==SCREEN) XFillRectangle(disp,win,gc,x1,y1,x2-x1+1,y2-y1+1);
    if (Cur_Out==PIXMAP) XFillRectangle(disp,map,gc,x1,y1,x2-x1+1,y2-y1+1); 
    if (Cur_Out==IMAGE)  for(i=0;i<XMaxRES*YMaxRES;i++) *(Idata+i)=f->work;
  }

VOID PutPixmap()
 {
  int x1,y1,x2,y2;

  x1=MinX; x2=MaxX;
  y1=MinY; y2=MaxY;

  if (Cur_Out==PIXMAP) XCopyArea(disp,map,win,gc,x1,y1,x2-x1+1,y2-y1+1,x1,y1); 
  if (Cur_Out==IMAGE) XPutImage(disp,win,gc,image,x1,y1,x1,y1,x2-x1+1,y2-y1+1);
 }

/*--------------------------------------------------------------------------*/

