/* draw a heightfield on the screen using X-Windows (X11 graphics) */
/* JPB 9/27/95 */

#include <stdio.h>                      /* fopen() sprintf() etc. */
#include <stdlib.h>                     /* strtod() */
#include <math.h>                       /* math stuff */
#include <string.h>                     /* strcpy() */
#include <ctype.h>                      /* tolower() */
#include "hcon.h"
#include "hf_io.h"
#define MAIN
#include "typy.h"

#define ESC '/033'

#define FALSE 0
#define TRUE 1
#define D double

#define Elmod(vv, xq, yq)  vv[(((y10+yq) % ysize) * xsize) + \
			   ((x10 +xq) % xsize)]  /* elmt of array */


struct Point3 {
 double x, y, z;                   /* three-dimensional point */
};

/* -------- X-Window graphics routines ----------- */
VOID Clear();
VOID X_Point(INT x,INT y);   /* drawable point */
VOID X_Poly(INT N,POINT2 point[]);
VOID X_FillPoly(INT N,POINT2 point[],INT CO,INT CV);
/* ------------------------------------------------- */

		/* draw two triangles joined along hypotenuse */
void tri3d(D x0, D z0,D x1,D z1,D y0,D y1,D y2,D y3, int s);
void drawhf(PTYPE *d1, int maxx, int maxz, int s, int t);
int graph_setup(int mode, int *xsize, int *ysize);
void graph_clear();   /* clear graphics screen */
void pr_help();              /* print help message */
void pr_vars(char *fname, int xsize, int ysize, int xgsize, int ygsize);
int getline(char s[], int lim);       /* read in one line */

D ex, ey, ez;      /* eye coordinate */
D zcenter;
D zfar, znear;
D zsf;      /* z coordinate scale factor */
D hs;      /* vertical (y) coordinate scale factor */
D ycorr;  /* y offset correction */
D zeps;   /* z epsilon */
int yoff;  /* screen coords y-offset */
int x10, y10;  /* hf size * 10 */
short sxmid;  /* center coordinates of view window (eg 640x480) */
short symid;
int nskip;        /* number of hf elements to skip */
int maxcol = 127;  /* maximum color index */
struct Point3 light;  /* position of light source */
D bright;           /* intensity of light  */
int maxcon;         /* maximize contrast? (boolean) */
int cmin, cmax;     /* max and min color index value in this image */
int erasewin = TRUE;       /* erase window between displays? */
int updatewin = TRUE;

int main (int argc, char *argv[])
{
  PTYPE *d1;      /* height field array */
  int xsize, ysize;  /* heightfield size */
  int xgsize, ygsize; /* screen size */
  char c;
  D eyinc;
  D lmag;         /* magnitude of a vector */
  int shader;     /* 0 wireframe 1 height 2 slope 3 lightsource */
  char instr[120];  /* input string */
  char fname[120];
  int tile = 0;
  int draw = TRUE;

  if (argc > 1) {
    strcpy(fname, argv[1]);
  } else strcpy(fname, "output.tga");

		/* set up graphics mode */

  printf("--------------------------------------------------------\n");
  printf("Xgrid v0.01  10/2/95 John Beale <beale@jump.stanford.edu>\n");
  printf("  --- draws wireframe or shaded heightfields on an X11 display.\n");
  printf("--------------------------------------------------------\n\n");
  if (readn_hf(&d1, (U *)&xsize,(U *)&ysize, fname) != 0) exit(1);
  nskip = (xsize / 50) + 1;
  printf("%s is a (%d x %d) heightfield.\n",fname,xsize,ysize);
  if (nskip != 1) {
      printf("Note: For faster preview, P%d has been set.\n",nskip);
      printf("To see %s in full resolution, enter P1.\n",fname);
    }
  xgsize = 600;
  ygsize = 400;
                      /* print text message, create window 'disp' */
  init_graph(xgsize,ygsize,maxcol+1);

  MinX = 0; MinY = 0;
  MaxX = GetMaxX(); 
  MaxY = GetMaxY();
  xgsize = MaxX;
  ygsize = MaxY;
  sxmid = xgsize / 2;  /* middle of screen in screen coords */
  symid = ygsize / 2;
  zsf = 0.3;      /* z coordinate scale factor */
  hs = 0.8;      /* vertical (y) coordinate scale factor */
  ycorr = 0.0;  /* y offset correction */
  zeps = 0.001;


	/* viewpoint 0,0,0 in +z direction */

  ex = 0.0; ey = -2; ez = -3;
  eyinc = 0.8;

  light.x = 3; light.y = 2; light.z = 0;   /* direction vector */
  lmag = sqrt(light.x*light.x + light.y*light.y + light.z*light.z);
  light.x /= lmag;
  light.y /= lmag;
  light.z /= lmag;

  bright = 0.8;  /* lightsource intensity */
  zcenter = 1;
  shader = 0;

  x10 = 10 * xsize;
  y10 = 10 * ysize;
  draw = TRUE;
  maxcon = FALSE;
  cmin = 0; cmax = maxcol;
  printf("Note: type 'h' for help.\n");
  /* ------------------------------------main interp. loop ------- */
  do {
    if (draw && updatewin) {
      MaxX = GetMaxX();      MaxY = GetMaxY();
      xgsize = MaxX;         ygsize = MaxY;
      sxmid = xgsize / 2;  /* middle of screen in screen coords */
      symid = ygsize / 2;
      yoff = (int) (symid*(ycorr + ey/zcenter)); 
      if (erasewin) Clear();
      drawhf(d1,xsize,ysize,shader,tile);
    }

    draw = TRUE;
    printf("xgrid> ");
    getline(instr,80);
    c = tolower(instr[0]);
    
    if (c == '+') hs *= 1.3;
    else if (c == '-') hs /= 1.3;
    else if (c == 'u') ey -= eyinc;
    else if (c == 'd') ey += eyinc;
    else if (c == 'b') ez -= 1.0;

    else if (c == '>') bright *= 1.2;
    else if (c == '<') bright /= 1.2;
    else if (c == 'm') maxcon = !maxcon;
    else if (c == 'w') updatewin = !updatewin;
    else if (c == 'v') {
       pr_vars(fname, xsize, ysize, xgsize, ygsize);
       draw = FALSE;
     }
    else if (c == 'l') {
      maxcon = FALSE;      /* reset contrast limits */
      sscanf(&instr[1],"%lf %lf %lf",&light.x,&light.y,&light.z);
      lmag = sqrt(light.x*light.x + light.y*light.y + light.z*light.z);
      light.x /= lmag;
      light.y /= lmag;
      light.z /= lmag;
    }
    else if (c == 'e') {
      maxcon = FALSE;      /* reset contrast limits */
      sscanf(&instr[1],"%lf %lf %lf",&ex,&ey,&ez);
      if (ez > -1.01) ez = -1.01;
    }
    else if (c == 'f') {
      ez += 0.5;
      if (ez >= -1.1) ez = -1.1;
    }
    else if ((c == 'h') || (c == '?')) {
      pr_help();
      draw = FALSE;
    }
    else if (c == 't') tile = !(tile);
    else if (c == 's') {
     maxcon = FALSE;
     shader = (int)atol(&instr[1]);
     shader--;
     if (shader > 3) shader = 3;
     if (shader < 0) shader = 0;
    }
    else if (c == 'r') {
     maxcon = FALSE;
     sscanf(&instr[1],"%s",fname);
     if (readn_hf(&d1, (U *)&xsize, (U *)&ysize, fname) != 0) {
       strcpy(fname,"output.tga");
       printf("Error reading %s\n",fname);
       if (readn_hf(&d1, (U *)&xsize, (U *)&ysize, fname) != 0) {
         printf("Error reading %s\n",fname);
         strcpy(fname,"previous image");
       }
     }
     printf("%s: (%d x %d)\n",fname,xsize,ysize);
    }
    else if (c == 'p') {
     nskip = (int)atol(&instr[1]);
     if (nskip < 1) nskip = 1;
    }
    else if (c == 'q') { /* quitting */ }
    else {
      printf("%s not understood.\n",instr);
      draw = FALSE;
    }

    zcenter = -ez;
    ycorr = 2*ey/(zcenter);
  } while (c != 'q');

    close_graph();          /* close window, de-allocate memory */  
  return(0);
}

void pr_vars(char *fname, int xsize, int ysize, int xgsize, int ygsize)
              /* print variable values */
{
   printf("eye coords:   %f %f %f\n",ex,ey,ez);
   printf("light vector: %f %f %f\n",light.x,light.y,light.z);
   printf("vertical scale: %f\n",hs);
   printf("lightsource I:  %f\n",bright);
   printf("contrast maximization is %s.\n",maxcon ? "ON" : "OFF");
   printf("color min,max: %d - %d\n",cmax,cmin);
   printf("grid spacing P%d",nskip);
   printf(" update is %s",updatewin ? "ON":"OFF");
   printf(" clear is %s.\n",erasewin ? "ON":"OFF");
   printf("%s is (%d x %d)  window: %d x %d\n",fname,xsize,ysize,
	  xgsize,ygsize);
}


void pr_help()              /* print help message */
{
    printf("----------- Xgrid Help ------------------------\n");
    printf("---  There are four shading options as follows:\n");
    printf("S1  wireframe   S2  height-shading\n");
    printf("S3  slope       S4  lightsource\n");
    printf("L x y z   vector to lightsource (affects S4 mode only)\n");
    printf("E x y z   position of viewer (eye)\n");
    printf("B/F   move eye position back/forward.\n");
    printf("U/D   move eye position up/down\n");
    printf("M     toggle 'maximize contrast' mode\n");
    printf("\n");
    printf(">/<   brighter/darker (S4 mode only)\n");
    printf("+/-   change vertical scaling of heightfield.\n");
    printf("T     tile heightfield in a 2x3 array\n");
    printf("Pn    draw only every nth line, for faster preview.\n");
    printf("V     print variables   W update?    H help\n");
    printf("R fn  read in a new file, eg   R output.pgm\n");
    printf("Q     quit\n");
    printf("\n");
}

int getline(char s[], int lim)       /* read in one line */
{ int c, i;
  c = 0;

  for (i=0; i<lim-1 && (c=getchar())!=EOF && c!='\n'; ++i)
     s[i] = c;
  if (c == '\n') {
        s[i] = c;
	++i;
      }
  s[i] = '\0';
  return(i);
}

/* =============================================================== */
/* project two triangles, given 3D coordinates, on a 2D viewscreen */
/* uses gobal vars:
 *   hs : vertical heightfield scaling factor
 *   ex,ey,ez : location of eye (viewpoint)
 *   symid : y coordinate midpoint
 *   ycorr : y correction factor
 *   yoff  : y offset in screen units 
 *   zcenter : center to 'rotate' appparent hf around
 *   maxcol  : maximum color value 
 *   bright  : lightsource intensity 
 * ===============================================================
 */

/*  shader: 0 = wireframe, 1 = height color, 2 = slope, 3 = lightsource */

void tri3d(D x0, D z0,D x1,D z1,D y0,D y1,D y2,D y3, int shader)
{
 D zmod0, zmod1; 
 D slope1;
 D slope2; 
 int c1, c2;  /* colors */
 D nx,ny,nz,nmag;   /* normal vector */
 D ctheta;          /* cos(theta)  theta = angle between light and normal */

 POINT2 p[3];       /* three sets of points in 2D */
 c2 = c1 = 0;            /* make compiler happy */ 
 
 if (shader == 1) {  /* height, before transform */
   c1 = maxcol*(y0 + y1 + y2 + y3)/4;
   c2 = c1;
 } 
 
 /* x0,y0,z0 should be in the range -1..1 */
 /* transform to eye coordinate (ex,ey,ez) */
 
 y0 *= -hs; y1 *= -hs; y2 *= -hs; y3 *= -hs;                          
 x0 -= ex; y0 -= ey; z0 -= ez;
 x1 -= ex; y1 -= ey; z1 -= ez;
 y2 -= ey; y3 -= ey; 
 if ((z1 <= zeps) || (z0 <= zeps)) return;  /* foreground clipping */

 if (shader == 2) {
   slope1 = atan2( (z0-z1)*(x1-x0),(z1-z0)*(y2-y0) );
   c1 = (int) ((maxcol>>1)*(0.5*slope1 + 1.8));
  
   slope2 = atan2( (z0-z1)*(x1-x0),(z1-z0)*(y1-y3) ); 
   c2 = (int) ((maxcol>>1)*(0.5*slope2 + 1.8));
   
 } else if (shader == 3) { /* compute normal, dot product with lightsource */

   nx = (z0-z1)*(y2-y0);
   ny = (z1-z0)*(x1-x0);
   nz = (x1-x0)*(y2-y1);
   nmag = sqrt(nx*nx + ny*ny + nz*nz);
   ctheta = (nx*light.x + ny*light.y + nz*light.z) / nmag;
   c1 = (int) (maxcol>>1)*(bright*(ctheta + 1.0));
   
   nx = -(z0-z1)*(y3-y1);
   /* ny = (z1-z0)*(x1-x0); */   /* duplicates line above */
   nz = (x0-x1)*(y3-y0);
   nmag = sqrt(nx*nx + ny*ny + nz*nz);
   ctheta = (nx*light.x + ny*light.y + nz*light.z) / nmag;
   c2 = (int) (maxcol>>1)*(bright*(ctheta + 1.0));

 } else if (shader == 0) {  /* shader = 0: wireframe */
   c1 = maxcol;  /* white, we hope. brightest color, anyway */
 }

 if (maxcon) {          /* if maximize_contrast is TRUE... */
   c1 = (int) (((D)maxcol/(cmax-cmin))*(c1-cmin));
   c2 = (int) (((D)maxcol/(cmax-cmin))*(c2-cmin));
 } 
 if (c1 < 0) c1 = 0;
 if (c1 > maxcol) c1 = maxcol;
 if (c2 < 0) c2 = 0;
 if (c2 > maxcol) c2 = maxcol;

 if (!maxcon) {
   if (c1 > cmax) cmax = c1;
   if (c1 < cmin) cmin = c1;
 }

 zmod0 = z0 * zsf;   /* z1 multiplied by z scale factor */
 zmod1 = z1 * zsf;
/*
 printf("zmod: %f %f\n",zmod0,zmod1);
  getch(); */
  /* sxmid, symid are midpoints of screen, eg sxmid=320 for 640x480 scn. */

 p[0].x = (int) (sxmid*x0/zmod0) + sxmid;  /* x,y,z 0 */
 p[0].y = (int) (symid*y0/zmod0) + symid + yoff;
 
 p[1].x = (int) (sxmid*x1/zmod1) + sxmid;  /* x,y,z 1 */
 p[1].y = (int) (symid*y1/zmod1) + symid + yoff;

 p[2].x = (int) (sxmid*x1/zmod0) + sxmid; /* x,y,z 2 */
 p[2].y = (int) (symid*y2/zmod0) + symid + yoff;
 
 if( ((p[0].x==p[1].x) && (p[0].y==p[1].y)) ||
     ((p[0].x==p[2].x) && (p[0].y==p[2].y)) ||
     ((p[1].x==p[2].x) && (p[1].y==p[2].y))  )  {
       X_Point(p[0].x,p[0].y);
   } else {
     if (shader == 0) {
       X_FillPoly(3,p,maxcol,0);
     }
     else if (shader == 1) {
       X_FillPoly(3,p,c1,maxcol/2+c1/2);
     } else 
       X_FillPoly(3,p,c1,c1);   
   }

 p[1].x = (int) (sxmid*x0/zmod0) + sxmid;  /* x,y,z 0 */
 p[1].y = (int) (symid*y0/zmod0) + symid + yoff;

 p[0].x = (int) (sxmid*x1/zmod1) + sxmid;  /* x,y,z 1 */
 p[0].y = (int) (symid*y1/zmod1) + symid + yoff;

 p[2].x = (int) (sxmid*x0/zmod1) + sxmid; /* x,y,z 3 */
 p[2].y = (int) (symid*y3/zmod1) + symid + yoff; 

if( ((p[0].x==p[1].x) && (p[0].y==p[1].y)) ||
     ((p[0].x==p[2].x) && (p[0].y==p[2].y)) ||
     ((p[1].x==p[2].x) && (p[1].y==p[2].y))  )  {
       X_Point(p[0].x,p[0].y);
   } else {
     if (shader == 0)  {
       X_FillPoly(3,p,maxcol,0);
     }
     else if (shader == 1) {
       X_FillPoly(3,p,c2,maxcol/2+c2/2);
     } else 
       X_FillPoly(3,p,c2,c2);
   }

} /* end tri3d() */

/* draw a heightfield, back to front, as a series of triangles */
/* uses global variable nskip as hf index increment. */

void drawhf(PTYPE *d1, int xsize, int ysize, int shader, int tile)
{
  int ix,iz,izi;
  int ymin, ymax, xmin, xmax;
  D xd2, yd2;
  D x0, y0, z0, x1, y1, z1;
  D y2, y3;

  
  xd2 = xsize / 2.0;
  yd2 = ysize / 2.0;
  if (tile) {
    ymin = 0;
    ymax = (2*ysize) - 1;
    xmin = -xsize + 1;
    xmax = 2*xsize -1;
  } else {
    ymin = 0;
    ymax = ysize - 2;
    xmin = 0;
    xmax = xsize-1;
  }
  if (!maxcon) {
    cmax = 0; 
    cmin = maxcol;     /* initialize global lims to extrema */
  }

		     /* loop over the two array indices */        
    for ( iz=ymin;iz<=ymax;iz += nskip) {
          izi = (ymax - iz) + ymin;
	  for ( ix=xmin;ix<xmax;ix+= nskip) {
	    x0 = (ix-xd2)/xd2; z0 = (izi-yd2)/yd2; 
	    x1 = (ix+nskip-xd2)/xd2; z1 = (izi-nskip-yd2)/yd2; 
	    y0 = (D) Elmod(d1,ix,iz);
	    y1 = (D) Elmod(d1,ix+nskip,iz+nskip);
	    y2 = (D) Elmod(d1,ix+nskip,iz);
	    y3 = (D) Elmod(d1,ix,iz+nskip);
	    tri3d(x0,z0,x1,z1,y0,y1,y2,y3,shader); 
	  } /* end for iz */
    } /* end for ix */
    Gflush();             /* flush events to X disp. */
} /* end drawhf() */

